@@ -46,8 +46,9 @@
 #'
 #' @references
 #'
-#' The Piecewise linear mixed model with abrupt and polynomial smooth change are described in van den Hout A, Muniz-Terrera G, Matthews F (2011). Smooth random change point models. Statistics in Medicine, 30(6):599-610.
-#' The saemix R package and SAEM algorithm are described in Comets E, Lavenu A, Lavielle MM (2017). Parameter estimation in nonlinear mixed effect models using saemix, an R implementation of the SAEM algorithm. Journal of Statistical Software, 80(3):1-41.
+#' Capuano AW, Wagner M. nlive: an R package to facilitate the application of the sigmoidal and random changepoint mixed models. BMC Medical Research Methodology. 2023;23(1):257.
+#' van den Hout A, Muniz-Terrera G, Matthews F. Smooth random change point models. Statistics in Medicine. 2011;30(6):599-610.
+#' Comets E, Lavenu A, Lavielle MM. Parameter estimation in nonlinear mixed effect models using saemix, an R implementation of the SAEM algorithm. Journal of Statistical Software. 2017;80(3):1-41.
 #'
 #' @examples
 #'
@@ -91,6 +92,7 @@
 #' @importFrom stats quantile
 #' @importFrom stats na.omit
 #' @importFrom sqldf sqldf
+#' @importFrom fastDummies dummy_cols
 #'
 #' @export
 ########
@@ -139,29 +141,89 @@ nlive.pmms <- function(dataset, ID, outcome, time, var.all = NULL,
   if (!is.null(start) & !is.vector(start))
     stop("The argument start must be a vector specifying the four initial values for the four main parameters in the following order: c(last level, initial level, midpoint, Hill slope).")
 
+  if (!is.null(traj.marg.group) & is.null(var.all) & is.null(var.last.level) & is.null(var.slope1) & is.null(var.slope2) & is.null(var.changepoint))
+    stop("The covariate in traj.marg.group should also be in at least one of these arguments: var.all, var.first.level, var.first.level, var.first.level.")
+  if (!is.null(traj.marg.group.val) & is.null(var.all) & is.null(var.last.level) & is.null(var.slope1) & is.null(var.slope2) & is.null(var.changepoint))
+    stop("The values in traj.marg.group.val are not used because no covariate specified in the argument traj.marg.group.")
 
   ## dataset
   dataset$ID      = na.omit(dataset[,ID])
   dataset$outcome = na.omit(dataset[,outcome])
   dataset$time    = na.omit(dataset[,time])
 
-  ## total nb of -unique- covariates
-  x          = unique(unlist(as.list(c(var.all, var.last.level, var.slope1, var.slope2, var.changepoint))))
+  ##########################################
+  ## accounting for categorical variables ##
+  ##########################################
+
+  ## var.all
+  data_subset     = subset(dataset, select = unique(unlist(as.list(c(var.all)))))
+  factor_cat_sup2 = as.list(sapply(data_subset, function(col) is.factor(col) && nlevels(col) > 2))
+  names_col       = names(which(factor_cat_sup2 == T))
+  ##  Dummy coding when nlevels(var.factor) > 2
+  if (length(names_col) == 0) { var.all2 = var.all } else if (length(names_col) > 0) {
+    data_subset2    = dummy_cols(data_subset, select_columns = names(which(factor_cat_sup2 == T)), remove_first_dummy = T, ignore_na = T, remove_selected_columns = T)
+    var.all2        = as.vector(colnames(data_subset2))
+  }
+
+  ## var.last.level
+  data_subset     = subset(dataset, select = unique(unlist(as.list(c(var.last.level)))))
+  factor_cat_sup2 = as.list(sapply(data_subset, function(col) is.factor(col) && nlevels(col) > 2))
+  names_col       = names(which(factor_cat_sup2 == T))
+  ##  Dummy coding when nlevels(var.factor) > 2
+  if (length(names_col) == 0) { var.last.level2 = var.last.level } else if (length(names_col) > 0) {
+    data_subset2    = dummy_cols(data_subset, select_columns = names(which(factor_cat_sup2 == T)), remove_first_dummy = T, ignore_na = T, remove_selected_columns = T)
+    var.last.level2 = as.vector(colnames(data_subset2))
+  }
+
+  ## var.slope1
+  data_subset     = subset(dataset, select = unique(unlist(as.list(c(var.slope1)))))
+  factor_cat_sup2 = as.list(sapply(data_subset, function(col) is.factor(col) && nlevels(col) > 2))
+  names_col       = names(which(factor_cat_sup2 == T))
+  ##  Dummy coding when nlevels(var.factor) > 2
+  if (length(names_col) == 0) { var.slope1_2 = var.slope1 } else if (length(names_col) > 0) {
+    data_subset2  = dummy_cols(data_subset, select_columns = names(which(factor_cat_sup2 == T)), remove_first_dummy = T, ignore_na = T, remove_selected_columns = T)
+    var.slope1_2  = as.vector(colnames(data_subset2))
+  }
+
+
+  ## var.slope2
+  data_subset     = subset(dataset, select = unique(unlist(as.list(c(var.slope2)))))
+  factor_cat_sup2 = as.list(sapply(data_subset, function(col) is.factor(col) && nlevels(col) > 2))
+  names_col       = names(which(factor_cat_sup2 == T))
+  ##  Dummy coding when nlevels(var.factor) > 2
+  if (length(names_col) == 0) { var.slope2_2 = var.slope2 } else if (length(names_col) > 0) {
+    data_subset2  = dummy_cols(data_subset, select_columns = names(which(factor_cat_sup2 == T)), remove_first_dummy = T, ignore_na = T, remove_selected_columns = T)
+    var.slope2_2  = as.vector(colnames(data_subset2))
+  }
+
+  ## var.changepoint
+  data_subset     = subset(dataset, select = unique(unlist(as.list(c(var.changepoint)))))
+  factor_cat_sup2 = as.list(sapply(data_subset, function(col) is.factor(col) && nlevels(col) > 2))
+  names_col       = names(which(factor_cat_sup2 == T))
+  ##  Dummy coding when nlevels(var.factor) > 2
+  if (length(names_col) == 0) { var.changepoint2 = var.changepoint } else if (length(names_col) > 0) {
+    data_subset2     = dummy_cols(data_subset, select_columns = names(which(factor_cat_sup2 == T)), remove_first_dummy = T, ignore_na = T, remove_selected_columns = T)
+    var.changepoint2 = as.vector(colnames(data_subset2))
+  }
+
+  ## definition of nb_cov & predictors & new data_subset with dummy variables (if any)
+  x          = unique(unlist(as.list(c(var.all2, var.last.level2, var.slope1_2, var.slope2_2, var.changepoint2))))
   nb_cov     = length(x)
   predictors = unique(x)
+  ##  New dataset with dummy var (if any)
+  data_subset     = subset(dataset, select = unique(unlist(as.list(c(var.all, var.last.level, var.slope1, var.slope2, var.changepoint)))))
+  factor_cols0 = as.list(sapply(data_subset, function(col) is.factor(col) && nlevels(col) > 2))
+  names_col    = names(which(factor_cols0 == T))
+  dataset2 = dummy_cols(dataset, select_columns = names_col, remove_first_dummy = T, ignore_na = T, remove_selected_columns = T)
 
+  ## remove NA from predictors (if any)
   if (length(predictors) > 0){
     for (i in 1:length(predictors)){
-      dataset = subset(dataset, is.na(dataset[,predictors[i]]) == F)
+      dataset2 = subset(dataset2, is.na(dataset2[,predictors[i]]) == F)
       i = i + 1
     }
   }
 
-  ## Values used throughout the code
-  first    = as.data.frame(dataset %>% group_by(ID) %>% filter(row_number(ID) == 1))
-  nb_indiv = dim(first)[1]
-  nb_line  = dim(dataset)[1]
-  nb_col   = dim(dataset)[2]
 
   ## Options
   # common to all plots #
@@ -250,66 +312,75 @@ nlive.pmms <- function(dataset, ID, outcome, time, var.all = NULL,
     }
     c(last.level, slope1, slope2, changepoint)
 
-    ## SPECIFICATION OF THE MATRIX OF COVARIATES ##
 
+    ## SPECIFICATION - MATRIX OF COVARIATES ##
+
+    # no predictors #
     if (nb_cov == 0) {
 
       mat_predictor = matrix(1, ncol=4, nrow=nb_cov)
 
-    } else if (is.null(var.all)==F & nb_cov == length(var.all)){
+      # only var.all #
+    } else if (is.null(var.all)==F & nb_cov == length(var.all2)){
 
       mat_predictor = matrix(1, ncol=4, nrow=nb_cov)
-      y = unique(unlist(as.list(var.all)))
+      y = unique(unlist(as.list(var.all2)))
       y = as.list(y)
       rownames(mat_predictor) = y
 
-    } else if (is.null(var.all)==F & nb_cov != length(var.all)){
+      #  var.all + other predictors #
+    } else if (is.null(var.all)==F & nb_cov != length(var.all2)){
 
-      mat.all = matrix(1, ncol=4, nrow=length(var.all))
-      y = unique(unlist(as.list(var.all)))
+      mat.all = matrix(1, ncol=4, nrow=length(var.all2))
+      y = unique(unlist(as.list(var.all2)))
       y = as.list(y)
       rownames(mat.all) = y
-      mat.tempo = matrix(0, ncol=4, nrow=nb_cov-length(var.all))
-      x = unique(unlist(as.list(c(var.last.level, var.slope1, var.slope2, var.changepoint))))
+      mat.tempo = matrix(0, ncol=4, nrow=nb_cov-length(var.all2))
+      x = unique(unlist(as.list(c(var.last.level2, var.slope1_2, var.slope2_2, var.changepoint2))))
       x = as.list(x)
       rownames(mat.tempo) = x
 
       for (i in 1:length(x)){
-        if (isTRUE(x[i] %in% var.last.level)){mat.tempo[i,1] = 1}
+        if (isTRUE(x[i] %in% var.last.level2)){mat.tempo[i,1] = 1}
       }
       for (i in 1:length(x)){
-        if (isTRUE(x[i] %in% var.slope1)){mat.tempo[i,2] = 1}
+        if (isTRUE(x[i] %in% var.slope1_2)){mat.tempo[i,2] = 1}
       }
       for (i in 1:length(x)){
-        if (isTRUE(x[i] %in% var.slope2)){mat.tempo[i,3] = 1}
+        if (isTRUE(x[i] %in% var.slope2_2)){mat.tempo[i,3] = 1}
       }
       for (i in 1:length(x)){
-        if (isTRUE(x[i] %in% var.changepoint)){mat.tempo[i,4] = 1}
+        if (isTRUE(x[i] %in% var.changepoint2)){mat.tempo[i,4] = 1}
       }
       mat_predictor = rbind(mat.all,mat.tempo)
+
+      # no var.all but other predictors #
     } else if (is.null(var.all)==T & nb_cov != 0){
+
       # x related to other than all parameters
+
       mat.tempo = matrix(0, ncol=4, nrow=nb_cov)
-      x = unique(unlist(as.list(c(var.last.level, var.slope1, var.slope2, var.changepoint))))
+      x = unique(unlist(as.list(c(var.last.level2, var.slope1_2, var.slope2_2, var.changepoint2))))
       x = as.list(x)
       rownames(mat.tempo) = x
 
       for (i in 1:length(x)){
-        if (isTRUE(x[i] %in% var.last.level)){mat.tempo[i,1] = 1}
+        if (isTRUE(x[i] %in% var.last.level2)){mat.tempo[i,1] = 1}
       }
       for (i in 1:length(x)){
-        if (isTRUE(x[i] %in% var.slope1)){mat.tempo[i,2] = 1}
+        if (isTRUE(x[i] %in% var.slope1_2)){mat.tempo[i,2] = 1}
       }
       for (i in 1:length(x)){
-        if (isTRUE(x[i] %in% var.slope2)){mat.tempo[i,3] = 1}
+        if (isTRUE(x[i] %in% var.slope2_2)){mat.tempo[i,3] = 1}
       }
       for (i in 1:length(x)){
-        if (isTRUE(x[i] %in% var.changepoint)){mat.tempo[i,4] = 1}
+        if (isTRUE(x[i] %in% var.changepoint2)){mat.tempo[i,4] = 1}
       }
       mat_predictor = rbind(mat.tempo)
     }
     mat_predictor
 
+
     ##
     saemix.model.cog = saemixModel(model = model_fct,
                                    psi0  = rbind(c(last.level  = last.level,
@@ -322,7 +393,7 @@ nlive.pmms <- function(dataset, ID, outcome, time, var.all = NULL,
                                    verbose = F)
 
     ##
-    write.table(dataset, file = "dataset.txt")
+    write.table(dataset2, file = "dataset.txt")
     dataset <- read.table('dataset.txt', header = TRUE, sep = "",dec=".")
     path = paste0(as.character(getwd()),"/dataset.txt")
     ##
@@ -333,7 +404,7 @@ nlive.pmms <- function(dataset, ID, outcome, time, var.all = NULL,
                             name.covariates = predictors,
                             verbose = F)
     ##
-    saemix.options = saemixControl(map = F,
+    saemix.options = saemixControl(map = F,print = F,
                                    nbiter.saemix = c(300,100), # nb of iterations for the exploration and smoothing phase
                                    nbiter.burn = 20,           # nb of iterations for burning
                                    nbiter.mcmc = c(5,5,5,0),   # nb of iterations in each kernel during the MCMC step
@@ -351,13 +422,13 @@ nlive.pmms <- function(dataset, ID, outcome, time, var.all = NULL,
     coef.PMM  = model_PMM@results@fixed.effects
 
     # calculation of p-values for the 4 parameters
-    tab = cbind(c(model_PMM@results@name.fixed, "error"),
-                c(round(model_PMM@results@fixed.effects,3), round(model_PMM@results@respar[model_PMM@results@indx.res],3)),
-                c(round(model_PMM@results@se.fixed,3),round(model_PMM@results@se.respar[model_PMM@results@indx.res],3)))
+    tab = cbind(c(model_PMM@results@name.fixed, "residual standard error"),
+                c(round(model_PMM@results@fixed.effects,6), round(model_PMM@results@respar[model_PMM@results@indx.res],6)),
+                c(round(model_PMM@results@se.fixed,6),round(model_PMM@results@se.respar[model_PMM@results@indx.res],6)))
     colnames(tab) = c("Parameter","Estimate","  SE")
     wstat = as.double(tab[,2])/as.double(tab[,3])
     pval  = rep(0, length(wstat))
-    pval2 = round(1 - normcdf(abs(wstat[1:length(pval)])), 4)
+    pval2 = round(1 - normcdf(abs(wstat[1:length(pval)])), 6)
     tab   = cbind(tab,"p-value" = pval2)
     tab = as.data.frame(tab)
     tab$`p-value` = ifelse(tab$`p-value` == "0", "P<.0001", tab$`p-value`)
@@ -690,7 +761,8 @@ nlive.pmms <- function(dataset, ID, outcome, time, var.all = NULL,
     }
 
   # output
-  print(tab)
-  cat("----------------------------------------------------\n The program took", round(cost[3],2), "seconds \n")
-  return(model.fit)
+    print(list(model.fit, tab))
+    cat("----------------------------------------------------\n The program took", round(cost[3],2), "seconds \n")
+    return(list(model.fit, tab))
+
 }
